# automatically generated by the FlatBuffers compiler, do not modify

# namespace: flat

import flatbuffers

class Float(object):
    __slots__ = ['_tab']

    # Float
    def Init(self, buf, pos):
        self._tab = flatbuffers.table.Table(buf, pos)

    # Float
    def Val(self): return self._tab.Get(flatbuffers.number_types.Float32Flags, self._tab.Pos + flatbuffers.number_types.UOffsetTFlags.py_type(0))

def CreateFloat(builder, val):
    builder.Prep(4, 4)
    builder.PrependFloat32(val)
    return builder.Offset()
