# automatically generated by the FlatBuffers compiler, do not modify

# namespace: flat

import flatbuffers

# /// A player that Rocket League treats as human, e.g. has a dedicated camera and can do training mode,
# /// but is actually controlled by a bot.
class PartyMemberBotPlayer(object):
    __slots__ = ['_tab']

    @classmethod
    def GetRootAsPartyMemberBotPlayer(cls, buf, offset):
        n = flatbuffers.encode.Get(flatbuffers.packer.uoffset, buf, offset)
        x = PartyMemberBotPlayer()
        x.Init(buf, n + offset)
        return x

    # PartyMemberBotPlayer
    def Init(self, buf, pos):
        self._tab = flatbuffers.table.Table(buf, pos)

def PartyMemberBotPlayerStart(builder): builder.StartObject(0)
def PartyMemberBotPlayerEnd(builder): return builder.EndObject()
