# automatically generated by the FlatBuffers compiler, do not modify

# namespace: flat

import flatbuffers

class Bool(object):
    __slots__ = ['_tab']

    # Bool
    def Init(self, buf, pos):
        self._tab = flatbuffers.table.Table(buf, pos)

    # Bool
    def Val(self): return self._tab.Get(flatbuffers.number_types.BoolFlags, self._tab.Pos + flatbuffers.number_types.UOffsetTFlags.py_type(0))

def CreateBool(builder, val):
    builder.Prep(1, 1)
    builder.PrependBool(val)
    return builder.Offset()
