# automatically generated by the FlatBuffers compiler, do not modify

# namespace: flat

class RumbleOption(object):
    No_Rumble = 0
    Default = 1
    Slow = 2
    Civilized = 3
    Destruction_Derby = 4
    Spring_Loaded = 5
    Spikes_Only = 6
    Spike_Rush = 7

