# automatically generated by the FlatBuffers compiler, do not modify

# namespace: flat

class GameMap(object):
    DFHStadium = 0
    Mannfield = 1
    ChampionsField = 2
    UrbanCentral = 3
    BeckwithPark = 4
    UtopiaColiseum = 5
    Wasteland = 6
    NeoTokyo = 7
    AquaDome = 8
    StarbaseArc = 9
    Farmstead = 10
    SaltyShores = 11
    DFHStadium_Stormy = 12
    DFHStadium_Day = 13
    Mannfield_Stormy = 14
    Mannfield_Night = 15
    ChampionsField_Day = 16
    BeckwithPark_Stormy = 17
    BeckwithPark_Midnight = 18
    UrbanCentral_Night = 19
    UrbanCentral_Dawn = 20
    UtopiaColiseum_Dusk = 21
    DFHStadium_Snowy = 22
    Mannfield_Snowy = 23
    UtopiaColiseum_Snowy = 24
    Badlands = 25
    Badlands_Night = 26
    TokyoUnderpass = 27
    Arctagon = 28
    Pillars = 29
    Cosmic = 30
    DoubleGoal = 31
    Octagon = 32
    Underpass = 33
    UtopiaRetro = 34
    Hoops_DunkHouse = 35
    DropShot_Core707 = 36
    ThrowbackStadium = 37
    ForbiddenTemple = 38
    RivalsArena = 39
    Farmstead_Night = 40
    SaltyShores_Night = 41
    NeonFields = 42

