# automatically generated by the FlatBuffers compiler, do not modify

# namespace: flat

class GameMode(object):
    Soccer = 0
    Hoops = 1
    Dropshot = 2
    Hockey = 3
    Rumble = 4
    Heatseeker = 5

