# automatically generated by the FlatBuffers compiler, do not modify

# namespace: flat

import flatbuffers

class BoostPad(object):
    __slots__ = ['_tab']

    @classmethod
    def GetRootAsBoostPad(cls, buf, offset):
        n = flatbuffers.encode.Get(flatbuffers.packer.uoffset, buf, offset)
        x = BoostPad()
        x.Init(buf, n + offset)
        return x

    # BoostPad
    def Init(self, buf, pos):
        self._tab = flatbuffers.table.Table(buf, pos)

    # BoostPad
    def Location(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(4))
        if o != 0:
            x = o + self._tab.Pos
            from .Vector3 import Vector3
            obj = Vector3()
            obj.Init(self._tab.Bytes, x)
            return obj
        return None

    # BoostPad
    def IsFullBoost(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(6))
        if o != 0:
            return self._tab.Get(flatbuffers.number_types.BoolFlags, o + self._tab.Pos)
        return 0

def BoostPadStart(builder): builder.StartObject(2)
def BoostPadAddLocation(builder, location): builder.PrependStructSlot(0, flatbuffers.number_types.UOffsetTFlags.py_type(location), 0)
def BoostPadAddIsFullBoost(builder, isFullBoost): builder.PrependBoolSlot(1, isFullBoost, 0)
def BoostPadEnd(builder): return builder.EndObject()
