# automatically generated by the FlatBuffers compiler, do not modify

# namespace: flat

class SeriesLengthOption(object):
    Unlimited = 0
    Three_Games = 1
    Five_Games = 2
    Seven_Games = 3

