# automatically generated by the FlatBuffers compiler, do not modify

# namespace: flat

class RespawnTimeOption(object):
    Three_Seconds = 0
    Two_Seconds = 1
    One_Seconds = 2
    Disable_Goal_Reset = 3

