# automatically generated by the FlatBuffers compiler, do not modify

# namespace: flat

class RenderType(object):
    DrawLine2D = 1
    DrawLine3D = 2
    DrawLine2D_3D = 3
    DrawRect2D = 4
    DrawRect3D = 5
    DrawString2D = 6
    DrawString3D = 7
    DrawCenteredRect3D = 8

