# automatically generated by the FlatBuffers compiler, do not modify

# namespace: flat

class GameMessage(object):
    NONE = 0
    PlayerStatEvent = 1
    PlayerSpectate = 2
    PlayerInputChange = 3

