# automatically generated by the FlatBuffers compiler, do not modify

# namespace: flat

class DemolishOption(object):
    Default = 0
    Disabled = 1
    Friendly_Fire = 2
    On_Contact = 3
    On_Contact_FF = 4

