# automatically generated by the FlatBuffers compiler, do not modify

# namespace: flat

import flatbuffers

class DropshotTile(object):
    __slots__ = ['_tab']

    @classmethod
    def GetRootAsDropshotTile(cls, buf, offset):
        n = flatbuffers.encode.Get(flatbuffers.packer.uoffset, buf, offset)
        x = DropshotTile()
        x.Init(buf, n + offset)
        return x

    # DropshotTile
    def Init(self, buf, pos):
        self._tab = flatbuffers.table.Table(buf, pos)

# /// The amount of damage the tile has sustained.
    # DropshotTile
    def TileState(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(4))
        if o != 0:
            return self._tab.Get(flatbuffers.number_types.Int8Flags, o + self._tab.Pos)
        return 0

def DropshotTileStart(builder): builder.StartObject(1)
def DropshotTileAddTileState(builder, tileState): builder.PrependInt8Slot(0, tileState, 0)
def DropshotTileEnd(builder): return builder.EndObject()
