# automatically generated by the FlatBuffers compiler, do not modify

# namespace: flat

import flatbuffers

# /// A console command which we will try to execute inside Rocket League.
# /// See https://github.com/RLBot/RLBot/wiki/Console-Commands for a list of known commands.
class ConsoleCommand(object):
    __slots__ = ['_tab']

    @classmethod
    def GetRootAsConsoleCommand(cls, buf, offset):
        n = flatbuffers.encode.Get(flatbuffers.packer.uoffset, buf, offset)
        x = ConsoleCommand()
        x.Init(buf, n + offset)
        return x

    # ConsoleCommand
    def Init(self, buf, pos):
        self._tab = flatbuffers.table.Table(buf, pos)

    # ConsoleCommand
    def Command(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(4))
        if o != 0:
            return self._tab.String(o + self._tab.Pos)
        return bytes()

def ConsoleCommandStart(builder): builder.StartObject(1)
def ConsoleCommandAddCommand(builder, command): builder.PrependUOffsetTRelativeSlot(0, flatbuffers.number_types.UOffsetTFlags.py_type(command), 0)
def ConsoleCommandEnd(builder): return builder.EndObject()
