# automatically generated by the FlatBuffers compiler, do not modify

# namespace: flat

class PlayerClass(object):
    NONE = 0
    RLBotPlayer = 1
    HumanPlayer = 2
    PsyonixBotPlayer = 3
    PartyMemberBotPlayer = 4

