# automatically generated by the FlatBuffers compiler, do not modify

# namespace: flat

class BoostOption(object):
    Normal_Boost = 0
    Unlimited_Boost = 1
    Slow_Recharge = 2
    Rapid_Recharge = 3
    No_Boost = 4

