# automatically generated by the FlatBuffers compiler, do not modify

# namespace: flat

import flatbuffers

# /// A psyonix bot, e.g. All Star bot
class PsyonixBotPlayer(object):
    __slots__ = ['_tab']

    @classmethod
    def GetRootAsPsyonixBotPlayer(cls, buf, offset):
        n = flatbuffers.encode.Get(flatbuffers.packer.uoffset, buf, offset)
        x = PsyonixBotPlayer()
        x.Init(buf, n + offset)
        return x

    # PsyonixBotPlayer
    def Init(self, buf, pos):
        self._tab = flatbuffers.table.Table(buf, pos)

    # PsyonixBotPlayer
    def BotSkill(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(4))
        if o != 0:
            return self._tab.Get(flatbuffers.number_types.Float32Flags, o + self._tab.Pos)
        return 0.0

def PsyonixBotPlayerStart(builder): builder.StartObject(1)
def PsyonixBotPlayerAddBotSkill(builder, botSkill): builder.PrependFloat32Slot(0, botSkill, 0.0)
def PsyonixBotPlayerEnd(builder): return builder.EndObject()
