# automatically generated by the FlatBuffers compiler, do not modify

# namespace: flat

import flatbuffers

class TeamInfo(object):
    __slots__ = ['_tab']

    @classmethod
    def GetRootAsTeamInfo(cls, buf, offset):
        n = flatbuffers.encode.Get(flatbuffers.packer.uoffset, buf, offset)
        x = TeamInfo()
        x.Init(buf, n + offset)
        return x

    # TeamInfo
    def Init(self, buf, pos):
        self._tab = flatbuffers.table.Table(buf, pos)

    # TeamInfo
    def TeamIndex(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(4))
        if o != 0:
            return self._tab.Get(flatbuffers.number_types.Int32Flags, o + self._tab.Pos)
        return 0

# /// number of goals scored.
    # TeamInfo
    def Score(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(6))
        if o != 0:
            return self._tab.Get(flatbuffers.number_types.Int32Flags, o + self._tab.Pos)
        return 0

def TeamInfoStart(builder): builder.StartObject(2)
def TeamInfoAddTeamIndex(builder, teamIndex): builder.PrependInt32Slot(0, teamIndex, 0)
def TeamInfoAddScore(builder, score): builder.PrependInt32Slot(1, score, 0)
def TeamInfoEnd(builder): return builder.EndObject()
